import * as schemas from '.';
/**
 * Sign Request (Base)
 *
 * A request to create a sign request object
 */
export interface SignRequestBase {
    /**
     * Indicates if the sender should receive a `prepare_url` in the response to complete document preparation via UI.
     * Example: true
     */
    is_document_preparation_needed?: boolean;
    /**
     * When specified, signature request will be redirected to this url when a document is signed.
     * Example: https://www.example.com
     */
    redirect_url?: string;
    /**
     * The uri that a signer will be redirected to after declining to sign a document.
     * Example: https://declined-redirect.com
     */
    declined_redirect_url?: string;
    /**
     * Disables the usage of signatures generated by typing (text).
     * Example: true
     * @default true
     */
    are_text_signatures_enabled?: boolean;
    /**
     * Subject of sign request email. This is cleaned by sign request. If this field is not passed, a default subject will be used.
     * Example: Sign Request from Acme
     */
    email_subject?: string;
    /**
     * Message to include in sign request email. The field is cleaned through sanitization of specific characters. However, some html tags are allowed. Links included in the message are also converted to hyperlinks in the email. The message may contain the following html tags including `a`, `abbr`, `acronym`, `b`, `blockquote`, `code`, `em`, `i`, `ul`, `li`, `ol`, and `strong`. Be aware that when the text to html ratio is too high, the email may end up in spam filters. Custom styles on these tags are not allowed. If this field is not passed, a default message will be used.
     * Example: Hello! Please sign the document below
     */
    email_message?: string;
    /**
     * Reminds signers to sign a document on day 3, 8, 13 and 18. Reminders are only sent to outstanding signers.
     * Example: true
     */
    are_reminders_enabled?: boolean;
    /**
     * The destination folder to place final, signed document and signing
     * log. Only `ID` and `type` fields are required. The root folder,
     * folder ID `0`, cannot be used.
     */
    parent_folder: schemas.FolderMini;
    /**
     * Name of the sign request.
     * Example: name
     */
    name?: string;
    /**
     * When a document contains sign related tags in the content, you can prefill them using this `prefill_tags` by referencing the 'id' of the tag as the `external_id` field of the prefill tag.
     */
    prefill_tags?: schemas.SignRequestPrefillTag[];
    /**
     * Set the number of days after which the created signature request will automatically expire if not completed. By default, we do not apply any expiration date on signature requests, and the signature request does not expire.
     * Example: 2
     */
    days_valid?: number;
    /**
     * This can be used to reference an ID in an external system that the sign request is related to.
     * Example: 123
     */
    external_id?: string;
    /**
     * Forces signers to verify a text message prior to viewing the document. You must specify the phone number of signers to have this setting apply to them.
     * Example: true
     */
    is_phone_verification_required_to_view?: boolean;
    /**
     * When a signature request is created from a template this field will indicate the id of that template.
     * Example: 123075213-af2c8822-3ef2-4952-8557-52d69c2fe9cb
     */
    template_id?: string;
}
